# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.

"""
This script holds all the functions for Webcenter Documents service Connections
(connections.xml) and any associated ADF Configation entries (adf-config.xml).
@author Dan Mullen
"""
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import Long
from java.lang import String
from java.lang import Exception
from javax.management import Attribute
import WebCenterHandlerHelper as helper
from javax.management import RuntimeMBeanException
from java.lang import Exception

# Code for detecting syntax error, especially missing colons, no need to translate
def doclibCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')


# Lists the parameters used by the Document Library
# service for configuration and maintenance of group spaces.
def listDocumentsSpacesProperties(appName,server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    spacesRoot = DLC().notSet
    adminUserName = DLC().notSet
    applicationName = DLC().notSet

    doclibConfigObj = getDoclibConfigObjName(appName)
    if doclibConfigObj is not None:
        try:
            spacesRoot = ora_mbs.getAttribute(doclibConfigObj,'SpacesRoot')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('getAttribute','SpacesRoot')
        try:
            adminUserName = ora_mbs.getAttribute(doclibConfigObj,'AdminUserName')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('getAttribute','AdminUserName')
        try:
            applicationName = ora_mbs.getAttribute(doclibConfigObj,'ApplicationName')
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('getAttribute','ApplicationName')

        printDoclibAdfConfigParams(spacesRoot, adminUserName, applicationName)
        primaryName = getDoclibPrimaryConnection(appName)
        printPrimaryDoclibConnection(primaryName)

    else:
        printErrorDoclibConfig('getDoclibConfigObjName','AdfConfig')

    helper.webcenterRestore()


# Sets the parameters that will be used by the Document Library
# service for configuration and maintenance of group spaces.
def setDocumentsSpacesProperties (appName, spacesRoot='*', adminUserName='*', applicationName='*',server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    doclibConfigObj = getDoclibConfigObjName(appName)
    if doclibConfigObj is not None:
        try:
            # Set up input parameters for setAttribute, do applicationName first
            # as that has server-side validation.  If any setter fails then
            # do not attempt to save the data, refresh it instead.
            userArg={}
            if applicationName != '*':
                userArg['ApplicationName'] = applicationName
            if spacesRoot != '*':
                userArg['SpacesRoot'] = spacesRoot
            if adminUserName != '*':
                userArg['AdminUserName'] = adminUserName
            exFromSet = helper.customSetAttributes(appName,doclibConfigObj,userArg)
            if exFromSet is None:
                helper.customSaveConfig(appName)
                helper.wcPrintMessage('INFORM_NEED_RESTART')
            else:
                raise exFromSet
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('setAttribute', ', '.join(userArg.keys()))
            helper.customRefreshConfig(appName)
    else:
        printErrorDoclibConfig('getDoclibConfigObjName','AdfConfig')

    helper.webcenterRestore()

    
# Removes the parameters that are used by the Document Library
# service for configuration and maintenance of group spaces.
def deleteDocumentsSpacesProperties(appName,server='',applicationVersion=''):
    #
    # This is really just a convenience routine.  The ASCTL implementation, DeleteSpacesPropertiesHelper,
    # just sets spacesRoot, adminUserName, and applicationName to null.  It doesn't really delete anything.
    # However, if desired we could get the adfConfig object and invoke removeConfig on it.
    setDocumentsSpacesProperties(appName, None, None, None,server,applicationVersion)


# File System specific JCR connection creation
def createJCRFileSystemConnection(appName, name, path, isPrimary=0, server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    if path == '' or path == ' ':
        printBadDoclibParamValue('path', path)
        return

    connectionSubType = DLC().fsSubType
    typeId = DLC().fsTypeId
    conArgs = {}
    subConArgs = {'BasePath':path}

    createJCRConnection(appName, name, conArgs, connectionSubType, typeId, subConArgs, isPrimary)
    helper.webcenterRestore()


# List connections that are File System specific
def listJCRFileSystemConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().fsSubType
    stringAttrs = DLC().fsStringAttrs
    nonStringAttrs = DLC().fsNonStringAttrs

    listJCRConnections(appName, connectionSubType, stringAttrs, baseNonStrAttrsFunc, nonStringAttrs, verbose, name)
    helper.webcenterRestore()
    

# Modify a File System specific connection
def setJCRFileSystemConnection(appName, name, path, isPrimary='', server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().fsSubType
    conArgs = {}
    subConArgs = {'BasePath':path}

    setJCRConnection(appName,name,connectionSubType, conArgs, subConArgs, isPrimary)
    helper.webcenterRestore()


# Content Server specific JCR connection creation
def createJCRContentServerConnection(appName, name, socketType, url='', \
                                     serverHost='', serverPort='',                \
                                     keystoreLocation='', keystorePassword='',    \
                                     privateKeyAlias='', privateKeyPassword='',   \
                                     webContextRoot='',                           \
                                     clientSecurityPolicy='',                     \
                                     cacheInvalidationInterval='',                \
                                     binaryCacheMaxEntrySize='',                  \
                                     adminUsername='', adminPassword='',          \
                                     extAppId='', timeout='', isPrimary=0,    \
                                     server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().stellentSubType
    typeId = DLC().stellentTypeId

    if extAppId == '':
        authenticationMethod = 'IDENTITY_PROPAGATION'
    else:
        authenticationMethod = 'EXTERNAL_APPLICATION_SERVICE'

    conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    if timeout != '':
        if long(timeout) <= 0:
            printBadDoclibParamValue('timeout', timeout)
            return
        conArgs['Timeout'] = Long(timeout)

    if socketType == '':
        printBadDoclibDefaultParam('socketType')
        return

    subConArgs={'CisSocketType':socketType}

    if socketType == 'socket' or socketType == 'socketssl':
        # with 'socket[ssl]' neither serverHost and serverPort can be defaulted
        if serverHost == '':
            printBadDoclibDefaultParam('serverHost', 'socketType=' + socketType)
            return
        if serverPort == '':
            printBadDoclibDefaultParam('serverPort', 'socketType=' + socketType)
            return

        subConArgs['ServerHost'] = serverHost
        subConArgs['ServerPort'] = serverPort

        if socketType == 'socket':
            if keystorePassword != '':
                # don't echo the original value
                keystorePassword = 'keystorePassword'
            if privateKeyPassword != '':
                # don't echo the original value
                privateKeyPassword = 'privateKeyPassword'
            doclibParamsShouldNotBeSet(url, keystoreLocation, privateKeyAlias, keystorePassword, privateKeyPassword)
        else:
            if keystoreLocation != '':
                subConArgs['KeystoreLocation'] = keystoreLocation
            if keystorePassword != '' and len(keystorePassword) > 0:
                subConArgs['KeystorePassword'] = jarray.array(list(keystorePassword),'c')
            if privateKeyAlias != '':
                subConArgs['PrivateKeyAlias'] = privateKeyAlias
            if privateKeyPassword != '' and len(privateKeyPassword) > 0:
                subConArgs['PrivateKeyPassword'] = jarray.array(list(privateKeyPassword),'c')

            doclibParamsShouldNotBeSet(url)
    elif socketType == 'web':
        if url == '':
            printBadDoclibDefaultParam('url', 'socketType=web')

        subConArgs['ServerWebUrl'] = url

        if keystorePassword != '':
            # don't echo the original value
            keystorePassword = 'keystorePassword'
        if privateKeyPassword != '':
            # don't echo the original value
            privateKeyPassword = 'privateKeyPassword'

        doclibParamsShouldNotBeSet(serverHost, serverPort, keystoreLocation, privateKeyAlias, keystorePassword, privateKeyPassword)
    elif socketType == 'jaxws':
        if url == '':
            printBadDoclibDefaultParam('url', 'socketType=jaxws')

        subConArgs['ServerWebUrl'] = url
        subConArgs['ClientSecurityPolicy'] = clientSecurityPolicy

        if keystorePassword != '':
            # don't echo the original value
            keystorePassword = 'keystorePassword'
        if privateKeyPassword != '':
            # don't echo the original value
            privateKeyPassword = 'privateKeyPassword'

        doclibParamsShouldNotBeSet(serverHost, serverPort, keystoreLocation, privateKeyAlias, keystorePassword, privateKeyPassword)
    else:
        printBadDoclibParamValue('socket', socketType)
        return

    if webContextRoot != '':
        subConArgs['WebContextRoot'] = webContextRoot

    if cacheInvalidationInterval != '':
        subConArgs['CacheInvalidationInterval'] = Long(cacheInvalidationInterval)

    if binaryCacheMaxEntrySize != '':
        subConArgs['BinaryCacheMaxEntrySize'] = Long(binaryCacheMaxEntrySize)

    if adminUsername != '':
        subConArgs['AdminUsername'] = adminUsername

    if adminPassword != ''  and len(adminPassword) > 0:
        subConArgs['AdminPassword'] = jarray.array(list(adminPassword),'c')

    createJCRConnection(appName, name, conArgs, connectionSubType, typeId, subConArgs, isPrimary)
    helper.webcenterRestore()


# List connections that are Content Server (Stellent) specific
def listJCRContentServerConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().stellentSubType
    stringAttrs = DLC().stellentStringAttrs
    nonStringAttrs = DLC().stellentNonStringAttrs

    listJCRConnections(appName, connectionSubType, stringAttrs, baseNonStrAttrsFunc, nonStringAttrs, verbose, name)
    helper.webcenterRestore()


# Modify a Content Server (Stellent) specific connection
def setJCRContentServerConnection(appName, name, socketType='*', url='*', \
                                  serverHost='*', serverPort='*',                   \
                                  keystoreLocation='*', keystorePassword='*',       \
                                  privateKeyAlias='*', privateKeyPassword='*',      \
                                  webContextRoot='*',                               \
                                  clientSecurityPolicy='*',                         \
                                  cacheInvalidationInterval='*',                    \
                                  binaryCacheMaxEntrySize='*',                      \
                                  adminUsername='*', adminPassword='*',             \
                                  extAppId='*', timeout='*', isPrimary='',          \
                                  server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().stellentSubType

    if extAppId == '':
        authenticationMethod = 'IDENTITY_PROPAGATION'
        conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    elif extAppId != '*':
        authenticationMethod = 'EXTERNAL_APPLICATION_SERVICE'
        conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    else:
        conArgs = {} # expAppId='*' so don't change either AuthenticationMethod or ExternalAppID

    if timeout == '':
        conArgs['Timeout'] = None
    elif timeout != '*':
        if long(timeout) <= 0:
            printBadDoclibParamValue('timeout', timeout)
            return
        conArgs['Timeout'] = Long(timeout)

    subConArgs={}

    #
    # TODO, implement logic for mutual exclusion and other parameter consistency checking
    #

    if socketType != '*':
        subConArgs['CisSocketType'] = socketType

    if serverHost != '*':
        subConArgs['ServerHost'] = serverHost

    if serverPort != '*':
        subConArgs['ServerPort'] = serverPort

    if keystoreLocation != '*':
        subConArgs['KeystoreLocation'] = keystoreLocation

    if keystorePassword != '*' and len(keystorePassword) > 0:
        subConArgs['KeystorePassword'] = jarray.array(list(keystorePassword),'c')

    if privateKeyAlias != '*':
        subConArgs['PrivateKeyAlias'] = privateKeyAlias

    if privateKeyPassword != '*' and len(privateKeyPassword) > 0:
        subConArgs['PrivateKeyPassword'] = jarray.array(list(privateKeyPassword),'c')

    if url != '*':
        subConArgs['ServerWebUrl'] = url

    if webContextRoot != '*':
        subConArgs['WebContextRoot'] = webContextRoot

    if clientSecurityPolicy != '*':
        subConArgs['ClientSecurityPolicy'] = clientSecurityPolicy

    if cacheInvalidationInterval != '*':
        subConArgs['CacheInvalidationInterval'] = Long(cacheInvalidationInterval)

    if binaryCacheMaxEntrySize != '*':
        subConArgs['BinaryCacheMaxEntrySize'] = Long(binaryCacheMaxEntrySize)

    if adminUsername != '*':
        subConArgs['AdminUsername'] = adminUsername

    if adminPassword != '*' and len(adminPassword) > 0:
        subConArgs['AdminPassword'] = jarray.array(list(adminPassword),'c')

    setJCRConnection(appName,name,connectionSubType, conArgs, subConArgs, isPrimary)
    helper.webcenterRestore()


# Portal specific JCR connection creation
def createJCRPortalConnection(appName, name, dataSource, extAppId='', timeout='', isPrimary=0, server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().portalSubType
    typeId = DLC().portalTypeId

    # DataSource *must* be specified
    if dataSource=='':
        printBadDoclibDefaultParam('dataSource')
        return

    if extAppId == '':
        authenticationMethod = 'IDENTITY_PROPAGATION'
    else:
        authenticationMethod = 'EXTERNAL_APPLICATION_SERVICE'

    conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    if timeout != '':
        if long(timeout) <= 0:
            printBadDoclibParamValue('timeout', timeout)
            return
        conArgs['Timeout'] = Long(timeout)

    subConArgs = {'DataSourceName':dataSource}

    createJCRConnection(appName, name, conArgs, connectionSubType, typeId, subConArgs, isPrimary)
    helper.webcenterRestore()


# List connections that are Portal specific
def listJCRPortalConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().portalSubType
    stringAttrs = DLC().portalStringAttrs
    nonStringAttrs = DLC().portalNonStringAttrs

    listJCRConnections(appName, connectionSubType, stringAttrs, baseNonStrAttrsFunc, nonStringAttrs, verbose, name)
    helper.webcenterRestore()


# Modify a Portal specific connection
def setJCRPortalConnection(appName, name, dataSource='*', extAppId='*', timeout='*', isPrimary='', server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().portalSubType

    if extAppId == '':
        authenticationMethod = 'IDENTITY_PROPAGATION'
        conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    elif extAppId != '*':
        authenticationMethod = 'EXTERNAL_APPLICATION_SERVICE'
        conArgs = {'AuthenticationMethod':authenticationMethod, 'ExternalAppID':extAppId}
    else:
        conArgs = {} # expAppId='*' so don't change either AuthenticationMethod or ExternalAppID

    if dataSource != '*':
        subConArgs = {'DataSourceName':dataSource}
    else:
        subConArgs = {}

    if timeout == '':
        conArgs['Timeout'] = None
    elif timeout != '*':
        if long(timeout) <= 0:
            printBadDoclibParamValue('timeout', timeout)
            return
        conArgs['Timeout'] = Long(timeout)

    setJCRConnection(appName,name,connectionSubType, conArgs, subConArgs, isPrimary)
    helper.webcenterRestore()


#
# Helper functions
#

# DocLib Constants
class DocLibConst:
    _instance = None
        
    def init(self):
        # Generic constants
        self.connectionType = 'JCR'
        self.notSet = helper.wcGetMessageFromResourceBundle('DL_ATTR_NOTSET')
        self.adfConfigName = 'oracle.adf.mbean.share.config.runtime.webcenter.doclib.AdfDoclibConfig'

        # File System Connection constants
        self.fsSubType = 'FileSystemConnection'
        self.fsTypeId = 'oracle.vcr.adapter.fs'
        self.fsStringAttrs = [(helper.wcGetMessageFromResourceBundle('DL_FILESYSTEM_ATTR_BASEPATH'),'BasePath')]
        self.fsNonStringAttrs = None

        # Stellent Connection constants
        self.stellentSubType = 'StellentConnection'
        self.stellentTypeId = 'oracle.stellent.jcr'
        self.stellentStringAttrs = [(helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_SOCKETTYPE'),'CisSocketType'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_HOSTNAME'),'ServerHost'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_PORTNUMBER'),'ServerPort'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_KEYSTORELOC'),'KeystoreLocation'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_PRIVATEKEYALIAS'),'PrivateKeyAlias'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_WEBURL'),'ServerWebUrl'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_WEBCONTEXTROOT'),'WebContextRoot'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_CLIENTSECURITYPOLICY'),'ClientSecurityPolicy'), \
                                    (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_ADMINUSERNAME'),'AdminUsername')]
        self.stellentNonStringAttrs = [(helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_CACHEINTERVAL'),'CacheInvalidationInterval'), \
                                       (helper.wcGetMessageFromResourceBundle('DL_STELLENT_ATTR_CACHEMAXSIZE'),'BinaryCacheMaxEntrySize')]

        # Portal Connection constants
        self.portalSubType = 'PortalConnection'
        self.portalTypeId = 'oracle.vcr.adapter.portal'
        self.portalStringAttrs = [(helper.wcGetMessageFromResourceBundle('DL_PORTAL_ATTR_DATASOURCE'),'DataSourceName')]
        self.portalNonStringAttrs = None


# Helper class for aggregating DocLib connection info
class DocLibCxn:
    __slots__ = ["connName", "mainConnObjName", "subConnType", "subConnObjName"]
##    def __init__(self, cName, mainObjName, subType, subObjName):
    def initialize(self, cName, mainObjName, subType, subObjName):
        self.connName = cName
        self.mainConnObjName = mainObjName
        self.subConnType = subType
        self.subConnObjName = subObjName

def DLC():
    if DocLibConst._instance == None:
        dlc = DocLibConst()
        dlc.init()
        DocLibConst._instance = dlc
        
    return DocLibConst._instance

def processDoclibPrimaryConnection(appName, name, isPrimary):
    if isPrimary is None or isPrimary == '':
        return None

    # Normalize isPrimary to be boolean
    if isPrimary == 'true':
        isPrimary = 1
    elif isPrimary == 'false':
        isPrimary = 0
    elif isPrimary != 1 and isPrimary != 0:
        printErrorDoclibConfig('setAttribute','PrimaryConnectionName')
        
    doclibConfigObj = getDoclibConfigObjName(appName)
    if doclibConfigObj is not None:
        userArg = None
        
        if isPrimary:
            # Set up input parameters for setAttribute
            userArg={'PrimaryConnectionName':name}
        else:
            primaryName = getDoclibPrimaryConnection(appName)
            if primaryName == name:
                # Set up input parameters for setAttribute
                userArg={'PrimaryConnectionName':None}

        if userArg is not None:
            for arg in userArg.items():
                try:
                    ora_mbs.setAttribute(doclibConfigObj,Attribute(arg[0],arg[1]))
                except Exception, ex:
                    printErrorDoclibConfig('setAttribute','PrimaryConnectionName')
                    helper.customRefreshConfig(appName)
                    return ex # Need to return if there is an exception so that no further sets are made
            if isPrimary:
                printPrimaryDoclibConnection(name)
            helper.customSaveConfig(appName) # only save if all the sets are working fine.
    else:
        printErrorDoclibConfig('setAttribute','PrimaryConnectionName')
        return None

def getDoclibPrimaryConnection(appName):
    doclibConfigObj = getDoclibConfigObjName(appName)
    if doclibConfigObj is not None:
        try:
            primaryName = ora_mbs.getAttribute(doclibConfigObj,'PrimaryConnectionName')
            return primaryName
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('getAttribute','PrimaryConnectionName')
        else:
            printErrorDoclibConfig('getAttribute','PrimaryConnectionName')
    return None

def getDoclibAttribute(appName, attr_name):
    doclibConfigObj = getDoclibConfigObjName(appName)
    if doclibConfigObj is not None:
        try:
            attr_value = ora_mbs.getAttribute(doclibConfigObj,attr_name)
            return attr_value
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorDoclibConfig('getAttribute',attr_name)
    else:
        printErrorDoclibConfig('getAttribute',attr_name)
    return None

# Generic code for creating a DocLib/JCR Content Connection
def createJCRConnection(appName, name, conArgs, connectionSubType, typeId, subConArgs, isPrimary):

    # Constants
    connectionType = DLC().connectionType

    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return

    # If a Connection MBean with that name already exists, abort the creation.
    if helper.connMBeanExists(ADFConnObjName,name,connectionType):
        helper.wcPrintErrorMessage('ERROR_CONNECTION_EXISTS',name)
        return
    else:
        objs = jarray.array([String(connectionType),String(name)],Object)
        strs = jarray.array(['java.lang.String','java.lang.String'],String)
        if helper.debugIsEnabled():
            connObjName = ora_mbs.invoke(ADFConnObjName,'createConnection',objs,strs)
            objNameStr = connObjName.toString()
        else:
            try:
                connObjName = ora_mbs.invoke(ADFConnObjName,'createConnection',objs,strs)
                objNameStr = connObjName.toString()
                # Set the main connection specific attributes
                exFromSet = setDoclibArgs(connObjName,conArgs,appName)
                if exFromSet is not None:
                    raise exFromSet
            except Exception, ex:
                helper.wcPrintExceptionMessage(ex.getMessage())
                helper.wcPrintErrorMessage('ERROR_CREATE_CONNECTION',name)
                helper.customRefresh(appName)
                return

    if helper.debugIsEnabled():
        print objNameStr

    # Create the repository specific 'SubConnection'
    objs = jarray.array([String(typeId)],Object)
    strs = jarray.array(['java.lang.String'], String)
    if helper.debugIsEnabled():
        subConnObjName = ora_mbs.invoke(connObjName, 'createJcrConnection', objs, strs)
        setDoclibArgs(subConnObjName,subConArgs,appName) 
        # helper.wcPrintMessage('CREATE_CONNECTION_SUCCESS',name)
        # helper.wcPrintMessage('INFORM_NEED_RESTART')
    else:
        try:        
            subConnObjName = ora_mbs.invoke(connObjName, 'createJcrConnection', objs, strs)
            exFromSet = setDoclibArgs(subConnObjName,subConArgs,appName) # Set the attributes of the subconnection
            if exFromSet is None:
                exFromPrimary = processDoclibPrimaryConnection(appName, name, isPrimary)
                if exFromPrimary is None: 
                    # Add a save operation so that the change persists in MDS
                    helper.customSave(appName) # Should we save the connection if setting the config errored out due to missing doclib config obj?
                    helper.wcPrintMessage('CREATE_CONNECTION_SUCCESS',name)
                    helper.wcPrintMessage('INFORM_NEED_RESTART')
                else:
                    raise exFromPrimary
            else:
                raise exFromSet
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_CREATE_CONNECTION',name)
            helper.customRefresh(appName)
            return


# Generic code for setting JCR Content connection attributes
def setJCRConnection(appName, name, connectionSubType, args, subArgs, isPrimary):

    names = []
    names.append(name)
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return
   
    objNamesDict = validateJCRConnectionNames(ADFConnObjName,appName,names,connectionSubType)

    if len(objNamesDict)>0:
        docLibConn = objNamesDict[name]
    
        objName = docLibConn.mainConnObjName
        subObjName = docLibConn.subConnObjName
        try:
            exFromArgs =setDoclibArgs(objName, args,appName)
            if exFromArgs is None:
                exFromSubArgs =setDoclibArgs(subObjName, subArgs,appName)
                if exFromSubArgs is None:
                    exFromConfig = processDoclibPrimaryConnection(appName, name, isPrimary)
                    if exFromConfig is None:
                        helper.customSave(appName)
                        helper.wcPrintMessage('INFORM_NEED_RESTART')
                    else:
                        raise exFromConfig
            else:
                raise exFromArgs
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.customRefresh(appName)
    else:
        printDoclibNoSuchConnection(name, connectionSubType)    


# Set attributes on a connection (or sub-connection)
def setDoclibArgs(objName,args,appName):
    for arg in args.items():
        try:
            ora_mbs.setAttribute(objName,Attribute(arg[0],arg[1]))
        except Exception, ex:            
            helper.wcPrintErrorMessage('ERROR_SET_ATTRIBUTE',arg[0])
            return ex
    return None

# Generic code for listing JCR Content connections
def listJCRConnections(appName,connectionSubType,stringAttrs,nonStrAttrsFunc,nonStrAttrs,verbose,name):
    
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return

    # names is a list.
    if name != '':
        # If the user provided a Connection for listing, validate the name and
        # get the Connection MBean's Object Name
        names=[name]
        objNamesDict = validateJCRConnectionNames(ADFConnObjName,appName,names,connectionSubType)
    else:
        # If not, get a list of all Connection MBean ObjectNames
        objNamesDict = getExistingJCRConnObjNames(ADFConnObjName, connectionSubType)
        if len(objNamesDict)<=0:
            helper.wcPrintErrorMessage('ERROR_NO_CONNECTION_TYPE',connectionSubType)
            return

    primaryName = getDoclibPrimaryConnection(appName)

    # list connections
    sortedKeys = objNamesDict.keys()
    sortedKeys.sort()
    for connName in sortedKeys:
        docLibConn = objNamesDict[connName]
        printJCRName(appName,docLibConn,stringAttrs,verbose,nonStrAttrsFunc,nonStrAttrs)
        if verbose and primaryName == connName:
            printPrimaryDoclibConnection(connName)

def getJCRSubConnObject(appName,connectionSubType,name):
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return None

    # names is a list.
    if name != '':
        # If the user provided a Connection for listing, validate the name and
        # get the Connection MBean's Object Name
        names=[name]
        objNamesDict = validateJCRConnectionNames(ADFConnObjName,appName,names,connectionSubType)
    else:
        helper.wcPrintErrorMessage('ERROR_NO_CONNECTION_TYPE',connectionSubType)
        return None

    #primaryName = getDoclibPrimaryConnection(appName)

    docLibConn = objNamesDict[name]
    return docLibConn.subConnObjName

def getJCRContentServerConnectionsObj(appName,name='',server='',applicationVersion=''):
    # Initialize WebCenter
    helper.webcenterInitialize(server,applicationVersion)
    connectionSubType = DLC().stellentSubType
    jcrSubConnObj = getJCRSubConnObject(appName, connectionSubType, name)
    helper.webcenterRestore()
    return jcrSubConnObj

def getJCRContentServerAttribute(appName,name,stringAttr,server='',applicationVersion=''):
    jcrCSConnObj = getJCRContentServerConnectionsObj(appName,name,server,applicationVersion)
    jcrCSStrAttr =  helper.getStrAttr(jcrCSConnObj,stringAttr)
    return jcrCSStrAttr

# Returns a hashmap of ObjectNames for all valid user-specified ADF JCR Connections
def validateJCRConnectionNames(ADFConnObjName,appName,names,connectionSubType):
    connectionType = DLC().connectionType
    validConnObjectNames={}
    invalidConnNames=[]
    existingConnObjNameDict = getExistingJCRConnObjNames(ADFConnObjName, connectionSubType)
    for n in names:
        if n in existingConnObjNameDict.keys():
            validConnObjectNames[n] = existingConnObjNameDict[n]
        else:
            invalidConnNames.append(n)

    invalidStr=helper.wcGetMessageFromResourceBundle('INFORM_INVALID_CONNECTION')
    for i in range(len(invalidConnNames)):
        invalidStr+=invalidConnNames[i]
        if i< len(invalidConnNames)-1:
            invalidStr+=', '
    if len(invalidConnNames)>0:
        print invalidStr

    return validConnObjectNames



#
# Return a dict (hashmap) where the key is the connection name and the value is
# a class consisting of the name, the connection subtype, and the two connObjNames
# (main connection and sub-connection)
#
def getExistingJCRConnObjNames(ADFConnObjName, connectionSubType):
    mbeanarray = helper.getMBeanArrayInADFConnections()
    subTypeSearchStr = 'JCRAdapterType=' + connectionSubType
    connObjNameDict={}

    # We cannot simply search for all the JCR connections like all the other services because
    # we'll get twice as many hits as we want (the main connection and the sub-connection).
    # So we need to search for connections and pair them up the corresponding subconnections.
    # In doing this we need to be careful not to pair "foo" (main) with "foo1" (sub).
    objs = jarray.array([String(DLC().connectionType)],Object)
    strs = jarray.array(['java.lang.String'],String)
    try:
        parentConnObjNameArray = ora_mbs.invoke(ADFConnObjName,'listConnectionMBeans',objs,strs)
        for connObjName in parentConnObjNameArray:
            name = ora_mbs.getAttribute(connObjName,'ConnectionName')
            nameSearch = 'name=' + name
            
            for i in range(len(mbeanarray)):
                objnamestr = mbeanarray[i].toString()
                nameIndex = objnamestr.find(nameSearch)
                if (objnamestr.find(subTypeSearchStr) >= 0) and (nameIndex >= 0):
                    possibleMatchLoc = nameIndex + len(nameSearch)
                    if (possibleMatchLoc == len(objnamestr)) or (objnamestr[possibleMatchLoc:possibleMatchLoc+1] == ','):
                        docLibConn = DocLibCxn()
                        docLibConn.initialize(name, connObjName, connectionSubType, mbeanarray[i])
                        connObjNameDict[name] = docLibConn
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_INVOKING_LISTCONNECTIONMBEANS')
        return None

    return connObjNameDict


# Get AdfDoclibConfig
def getDoclibConfigObjName(appName):
    # Get/create DocLib Config MBean's Object Name
    return helper.findOrCreateConfigObjName(appName, DLC().adfConfigName)


def baseNonStrAttrsFunc(objName, nonStrAttrs):
    if nonStrAttrs is None:
        return;

    for item in nonStrAttrs:
        try:
            attrLabel=item[0]
            obj = ora_mbs.getAttribute(objName,item[1])
            if obj is None:
                attrValue=DLC().notSet
            else:
                attrValue=str(obj)
            print attrLabel+ ' ' +attrValue
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',item[1])


def printJCRName(appName,docLibConn,stringAttrs,verbose,nonStrAttrsFunc,nonStrAttrs):
    try:
        connectionSubType = docLibConn.subConnType
        objName = docLibConn.mainConnObjName
        subObjName = docLibConn.subConnObjName

### TODO ####
# Add Authentication Method back into this list.  A bug fix is necessary before it will work in all cases.
        baseStrAttrs = [(helper.wcGetMessageFromResourceBundle('DL_ATTR_CONNECTIONNAME'),'ConnectionName'), \
                        (helper.wcGetMessageFromResourceBundle('DL_ATTR_CONNECTIONTYPE'),'ConnectionType'), \
                        (helper.wcGetMessageFromResourceBundle('DL_ATTR_EXTAPPID'),'ExternalAppID')]
                     #, 'Authentication Method: ':'AuthenticationMethod')]
        baseNonStrAttrs = [(helper.wcGetMessageFromResourceBundle('DL_ATTR_TIMEOUT'),'Timeout')]

        helper.printName(appName, objName, DLC().connectionType, baseStrAttrs, verbose, baseNonStrAttrsFunc, baseNonStrAttrs)
        
        if verbose==1:
            helper.printStrAttrs(subObjName,stringAttrs)
            nonStrAttrsFunc(subObjName,nonStrAttrs)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConnectionName')


def printBadDoclibDefaultParam(paramName, parentParam=''):
    if parentParam != '':
        context = helper.wcGetMessageFromResourceBundle('DL_ERROR_NO_DEFAULT_PARAM_CONTEXT', parentParam)
    else:
        context = ''

    helper.wcPrintErrorMessage('DL_ERROR_NO_DEFAULT_PARAM', paramName+','+context)
    

def printBadDoclibParamValue(paramName, paramValue):
    helper.wcPrintErrorMessage('DL_ERROR_BAD_PARAM_VALUE', str(paramValue)+','+paramName)


def printErrorDoclibConfig(oper,arg):
    helper.wcPrintErrorMessage('DL_ERROR_PERFORMING_OP', oper+','+arg)


def printPrimaryDoclibConnection(connName):
    helper.wcPrintMessage('DL_INFO_PRIMARY_CONNECTION_IS', connName)


def printDoclibAdfConfigParams(spacesRoot, adminUserName, applicationName):
    helper.wcPrintMessage('DL_INFO_SPACES_CONTAINER_IS', spacesRoot)
    helper.wcPrintMessage('DL_INFO_REPOS_ADMIN_IS', adminUserName)
    helper.wcPrintMessage('DL_INFO_APP_NAME_IS', applicationName)

def printDoclibNoSuchConnection(name, connectionSubType):
    helper.wcPrintErrorMessage('DL_ERROR_NO_NAME_TYPE', name+','+connectionSubType)


def doclibParamsShouldNotBeSet(*params):
    for param in params:
        if param != '':
            helper.wcPrintMessage('DL_WARNING_PARAM_NOT_VALID', param)

def getJCRConnectionAttribute(appName, connectionName, connectionSubType, attributeType, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    attrVal = None
    jcrSubObj = getJCRSubConnObject(appName,connectionSubType,connectionName)
    if jcrSubObj is None :
        return None
    try:
        attrVal = ora_mbs.getAttribute(jcrSubObj,attributeType)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',attributeType)
    helper.webcenterRestore()
    return attrVal

def getJCRConnectionsBySubType(appName, connectionSubType, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    if ADFConnObjName is None:
        return None
    objNamesDict = getExistingJCRConnObjNames(ADFConnObjName, connectionSubType)
    if len(objNamesDict)<=0:
        helper.wcPrintErrorMessage('ERROR_NO_CONNECTION_TYPE',connectionSubType)
        return None
    helper.webcenterRestore()
    return objNamesDict

def getJCRPrimaryAttribute(appName, docLibObj, attributeType, server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    
    objName = docLibObj.mainConnObjName
    try:
        attrVal = ora_mbs.getAttribute(objName,attributeType)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',attributeType)

    helper.webcenterRestore()
    return attrVal

